function [sbestbest,gbestbest,maxvarbestbest,Wbest,nvarfevals,nfevals] =...
    GoptViaIopt(nfactors,n,nstarts)
%
% Function to set up the problem and apply coordinate exchange to the
% w values of the weight matrix.  For each set of (w_1,w_2) values to be
% evaluated, the I_(lamda) optimal design is first computed (via the call to
% LuciaIOPT2_grid_CJN) and the the maximum variance is determined for the
% resulting I_(lamda)-optimal design
%
global fevals;
global varfevals;
varfevals=0;
fevals=0;
model = 'quadratic';
q=nfactors;
ngridpoints = 5;
range = .2;
halfrange = range/2;
stepsize = range/(ngridpoints - 1);
wvals = [.5,.5;.74,.58;.79,.65;.83,.70;.85,.74];
w1val = wvals(q,1);
w2val = wvals(q,2);
gbestbest = 0;
Wbest=zeros(1,2);
%1-factor (D-optimal: .67 - .67):
%2-factor (D-optimal: .74 - .58):
%3-factor (D-optimal: .79 - .65):
%4-factor (D-optimal: .83 - .70):
%5-factor (D-optimal: .85 - .74):
for w1=(w1val-halfrange):stepsize:(w1val+halfrange)
    for w2=(w2val-halfrange):stepsize:(w2val+halfrange)
        % for w3=0.4:0.05:w1
        if q==1
            W=[1 0 w1;0 w1 0;w1 0 w2];end
        if q>1
            %aux=q*(q-1)/2;
            aux_matrix=w2*ones(q,q);
            aux_matrix(1:q+1:q*q)=w1;
            W=[[1 zeros(1,q) zeros(1,q*(q-1)/2) w1*ones(1,q)];
                [zeros(q,1) w1*eye(q) zeros(q,q*(q-1)/2) zeros(q,q)];
                [zeros(q*(q-1)/2,1) zeros(q*(q-1)/2,q) w2*eye(q*(q-1)/2) zeros(q*(q-1)/2,q)];
                [w1*ones(q,1) zeros(q,q) zeros(q,q*(q-1)/2) aux_matrix]];
        end;
        
        [sbest,maxvarbest,gbest] = LuciaIOPT2_grid_CJN(W, model, n, q, nstarts);
        % display(sbest);
        % display([maxvarbest,gbest])
        
        if gbest > gbestbest
            gbestbest = gbest;
            sbestbest = sbest;
            Wbest = [w1,w2];
            maxvarbestbest=maxvarbest;
        end
        %end
        %    end
    end
    nfevals=fevals;
    nvarfevals=varfevals;
%     display(sbest)
%     display(maxvarbest)
%     display(gbest)
%     display(Wbest);
%     display(fevals)
%     display(varfevals)
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function LuciaIOPT2_grid_CJN
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [sbest,maxvarbest,gbest] = ...
    LuciaIOPT2_grid_CJN(W, model, n, q, nstarts)
% Computes the exact I_(lamda)-Optimal design, given W
global fevals;

p = (q+1)*(q+2)/2;
gbest=0;
for istart = 1:nstarts    
    s=(2.*rand(n,q) -1);
    X = zeros(n,p);
    for i=1:n
        X(i,:) = x2fx(s(i,:),model);
    end;
    cbest = .1e20;
    c0 = trace((X'*X)\W);
    while abs(c0-cbest)>1.000e-8
        cbest = c0;
        for i = 1:n
            for ivar = 1:q
                for xpt = -1:0.05:1
                    stemp = s(i, ivar);
                    Xtemp = X(i,:);
                    s(i,ivar) = xpt;
                    X(i,:) = x2fx(s(i,:), model);
                    if rank(X,.1e-5) >= p
                        cnew = trace((X'*X)\W);
                        fevals=fevals+1;
                        if cnew < c0
                            c0 = cnew;
                        else
                            s(i,ivar) = stemp;
                            X(i,:) = Xtemp;
                        end
                    else
                        s(i,ivar) = stemp;
                        X(i,:) = Xtemp;
                    end
                end
            end
        end
    end
    
    [maxvar,~] = gridmaxvar(s,model);
    
    geff = p/maxvar;
    if geff > gbest
        sbest = s;
        maxvarbest=maxvar;
        gbest = geff;
    end
    
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function gridmaxvar
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [maxvar,xmax] = gridmaxvar(s,model)
% finds maximum variance using gridsize = 0.5 (5 pt grid)
global varfevals
nf = size(s,2);
gridpts = 5;
X = x2fx(s,model);
[n,~] = size(X);
if rank(X,.000001) < size(X,2)
    maxvar = .1e20;
    xmax = 0;
    return
end
D = invmat(X'*X);
gridpts = (fullfact(ones(1,nf)*gridpts)-1)/((gridpts-1)/2)-1;
ngridpts = size(gridpts,1);
maxvar = 0;
for ipt = 1:ngridpts
    xvec = gridpts(ipt,:);
    fx = x2fx(xvec,model)';
    var = n*fx'*D*fx;
    varfevals= varfevals + 1;
    if var > maxvar
        maxvar = var;
        xmax = xvec;
    end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function invmat
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B =  invmat(A)
% Purpose is simply to avoid the use of "inv" function
% in the main routine which gives an annoying warning
B = inv(A);
end
